'use strict';

/* jshint node: true, esnext: true, loopfunc: true, undef: true, unused: true */

var arrayUtils = require('zero-lang/array');
var objectUtils = require('zero-lang/object');

var escape = encodeURIComponent;
var unescape = function unescape(s) {
    return decodeURIComponent(s.replace(/\+/g, ' '));
};

module.exports = {
    /*
     * @description: override default encoding method
     * @syntax: querystring.escape(str)
     * @param: {String} str, unescaped string.
     * @return: {String} escaped string.
     */
    escape: escape,

    /*
     * @description: override default decoding method
     * @syntax: querystring.unescape(str)
     * @param: {String} str, escaped string.
     * @return: {String} unescaped string.
     */
    unescape: unescape,

    parse: function parse(qs, sep, eq) {
        /*
         * @description: accept query strings and return native javascript objects.
         * @syntax: querystring.parse(str)
         * @param: {String} str, query string to be parsed.
         * @return: {Object} parsed object.
         */
        qs = qs.replace(/^\?/, ''); // remove the heading '?'
        sep = sep || '&';
        eq = eq || '=';
        var tuple = undefined;
        var obj = {};
        var pieces = qs.split(sep);

        arrayUtils.each(pieces, function (elem) {
            tuple = elem.split(eq);
            if (tuple.length > 0) {
                obj[unescape(tuple.shift())] = unescape(tuple.join(eq));
            }
        });
        return obj;
    },
    stringify: function stringify(obj, c) {
        /*
         * @description: converts an arbitrary value to a query string representation.
         * @syntax: querystring.stringify(obj)
         * @param: {object} obj, object to be stringified
         * @return: {String} query string.
         */
        var qs = [];
        var s = c && c.arrayKey ? true : false;

        objectUtils.forIn(obj, function (value, key) {
            if (arrayUtils.isArray(value)) {
                arrayUtils.each(value, function (elem) {
                    qs.push(escape(s ? key + '[]' : key) + '=' + escape(elem));
                });
            } else {
                qs.push(escape(key) + '=' + escape(value));
            }
        });
        return qs.join('&');
    }
};